@@ -5,9 +5,7 @@ import networkx as nx
 from dataclasses import dataclass
 
 # Imported here so we can call from force import compute_bond_forces__fortran
-from force_kernels import (  # noqa: F401
-    cbf as compute_bond_forces__fortran
-)
+from force_kernels import cbf as compute_bond_forces__fortran  # noqa: F401
 from force_kernels import (  # noqa: F401
     caf as compute_angle_forces__fortran,
 )
@@ -25,6 +23,12 @@ from force_kernels import (  # noqa: F401
 )
 
 
+@dataclass  # might not be needed
+class Dielectric_type:
+    atom_1: str
+    dielectric_value: float
+
+
 @dataclass
 class Bond:
     """Dataclass representing a single two-particle bond type
@@ -57,6 +61,7 @@ class Bond:
     strength : float
         Harmonic bond strength coefficient (spring constant).
     """
+
     atom_1: str
     atom_2: str
     equilibrium: float
@@ -111,6 +116,7 @@ class Angle(Bond):
     Bond :
         Two-particle bond type dataclass
     """
+
     atom_3: str
 
 
@@ -205,6 +211,7 @@ class Dihedral:
     ----------
     Bore et al. J. Chem. Theory Comput., 14(2): 1120–1130, 2018.
     """
+
     atom_1: str
     atom_2: str
     atom_3: str
@@ -251,6 +258,7 @@ class Chi:
     hymd.hamiltonian.DefaultWithChi :
         Interaction energy functional using :math:`\\chi`-interactions.
     """
+
     atom_1: str
     atom_2: str
     interaction_energy: float
@@ -368,12 +376,8 @@ def prepare_bonds_old(molecules, names, bonds, indices, config):
                     name_j = bond_graph.nodes()[j]["name"]
 
                     for b in config.bonds:
-                        match_forward = (
-                            name_i == b.atom_1 and name_j == b.atom_2
-                        )
-                        match_backward = (
-                            name_i == b.atom_2 and name_j == b.atom_1
-                        )
+                        match_forward = name_i == b.atom_1 and name_j == b.atom_2
+                        match_backward = name_i == b.atom_2 and name_j == b.atom_1
                         if match_forward or match_backward:
                             bonds_2.append(
                                 [
@@ -456,7 +460,61 @@ def prepare_bonds_old(molecules, names, bonds, indices, config):
     return bonds_2, bonds_3, bonds_4, bb_index
 
 
-def prepare_bonds(molecules, names, bonds, indices, config):
+def prepare_index_based_bonds(molecules, topol):
+    bonds_2 = []
+    bonds_3 = []
+    bonds_4 = []
+
+    different_molecules = np.unique(molecules)
+    for mol in different_molecules:
+        resid = mol + 1
+        top_summary = topol["system"]["molecules"]
+        resname = None
+        test_mol_number = 0
+        for molname in top_summary:
+            test_mol_number += molname[1]
+            if resid <= test_mol_number:
+                resname = molname[0]
+                break
+
+        if resname is None:
+            break
+
+        if "bonds" in topol[resname]:
+            first_id = np.where(molecules == mol)[0][0]
+            for bond in topol[resname]["bonds"]:
+                index_i = bond[0] - 1 + first_id
+                index_j = bond[1] - 1 + first_id
+                # bond[2] is the bond type, inherited by the itp format. Not used
+                equilibrium = bond[3]
+                strength = bond[4]
+                bonds_2.append([index_i, index_j, equilibrium, strength])
+
+        if "angles" in topol[resname]:
+            first_id = np.where(molecules == mol)[0][0]
+            for angle in topol[resname]["angles"]:
+                index_i = angle[0] - 1 + first_id
+                index_j = angle[1] - 1 + first_id
+                index_k = angle[2] - 1 + first_id
+                # angle[3] is the angle type, inherited by the itp format. Not used
+                equilibrium = np.radians(angle[4])
+                strength = angle[5]
+                bonds_3.append([index_i, index_j, index_k, equilibrium, strength])
+
+        if "dihedrals" in topol[resname]:
+            first_id = np.where(molecules == mol)[0][0]
+            for angle in topol[resname]["dihedrals"]:
+                index_i = angle[0] - 1 + first_id
+                index_j = angle[1] - 1 + first_id
+                index_k = angle[2] - 1 + first_id
+                index_l = angle[3] - 1 + first_id
+                dih_type = angle[4]
+                coeff = angle[5]
+                bonds_4.append([index_i, index_j, index_k, index_l, coeff, dih_type, 0])
+    return bonds_2, bonds_3, bonds_4
+
+
+def prepare_bonds(molecules, names, bonds, indices, config, topol=None):
     """Rearrange the bond information for usage in compiled Fortran kernels
 
     Restructures the lists resulting from the execution of
@@ -538,9 +596,14 @@ def prepare_bonds(molecules, names, bonds, indices, config):
         connectivity information in the structure/topology input file and the
         bonded types specified in the configuration file.
     """
-    bonds_2, bonds_3, bonds_4, bb_index = prepare_bonds_old(
-        molecules, names, bonds, indices, config
-    )
+    if topol is not None:
+        bonds_2, bonds_3, bonds_4 = prepare_index_based_bonds(
+            molecules, topol
+        )
+    else:
+        bonds_2, bonds_3, bonds_4, bb_index = prepare_bonds_old(
+            molecules, names, bonds, indices, config
+        )
 
     # Bonds
     bonds_2_atom1 = np.empty(len(bonds_2), dtype=int)
@@ -565,7 +628,6 @@ def prepare_bonds(molecules, names, bonds, indices, config):
         bonds_3_atom3[i] = b[2]
         bonds_3_equilibrium[i] = b[3]
         bonds_3_strength[i] = b[4]
-
     # Dihedrals
     bonds_4_atom1 = np.empty(len(bonds_4), dtype=int)
     bonds_4_atom2 = np.empty(len(bonds_4), dtype=int)
@@ -585,12 +647,28 @@ def prepare_bonds(molecules, names, bonds, indices, config):
         bonds_4_atom4[i] = b[3]
         bonds_4_coeff[i] = np.resize(b[4], (number_of_coeff, len_of_coeff))
         bonds_4_type[i] = b[5]
-    bonds_4_last[bb_index] = 1
+        if topol is not None:
+            bonds_4_last[i] = b[6]
+    if topol is None:
+        bonds_4_last[bb_index] = 1
 
     return (
-        bonds_2_atom1, bonds_2_atom2, bonds_2_equilibrium, bonds_2_strength,
-        bonds_3_atom1, bonds_3_atom2, bonds_3_atom3, bonds_3_equilibrium, bonds_3_strength,  # noqa: E501
-        bonds_4_atom1, bonds_4_atom2, bonds_4_atom3, bonds_4_atom4, bonds_4_coeff, bonds_4_type, bonds_4_last,  # noqa: E501
+        bonds_2_atom1,
+        bonds_2_atom2,
+        bonds_2_equilibrium,
+        bonds_2_strength,
+        bonds_3_atom1,
+        bonds_3_atom2,
+        bonds_3_atom3,
+        bonds_3_equilibrium,
+        bonds_3_strength,  # noqa: E501
+        bonds_4_atom1,
+        bonds_4_atom2,
+        bonds_4_atom3,
+        bonds_4_atom4,
+        bonds_4_coeff,
+        bonds_4_type,
+        bonds_4_last,  # noqa: E501
     )
 
 
@@ -647,7 +725,7 @@ def compute_angle_forces__plain(f_angles, r, bonds_3, box_size):
 
         cosphi = np.dot(ea, ec)
         theta = np.arccos(cosphi)
-        xsinph = 1.0 / np.sqrt(1.0 - cosphi ** 2)
+        xsinph = 1.0 / np.sqrt(1.0 - cosphi**2)
 
         d = theta - theta0
         f = -k * d
@@ -721,14 +799,14 @@ def compute_dihedral_forces__plain(f_dihedrals, r, bonds_4, box_size):
 def dipole_forces_redistribution(
     f_on_bead, f_dipoles, trans_matrices, a, b, c, d, type_array, last_bb
 ):
-    """Redistribute electrostatic forces calculated from topologically 
+    """Redistribute electrostatic forces calculated from topologically
     reconstructed ghost dipole point charges to the backcone atoms of the protein.
     """
     f_on_bead.fill(0.0)
     for i, j, k, l, fd, matrix, dih_type, is_last in zip(
         a, b, c, d, f_dipoles, trans_matrices, type_array, last_bb
     ):
-        if dih_type ==1:
+        if dih_type == 1:
             sum_force = fd[0] + fd[1]
             diff_force = fd[0] - fd[1]
             f_on_bead[i] += matrix[0] @ diff_force  # Atom A
