@@ -14,8 +14,8 @@ import sympy
 
 
 class Hamiltonian:
-    """Interaction energy functional superclass
-    """
+    """Interaction energy functional superclass"""
+
     def __init__(self, config):
         """Constructor
 
@@ -39,20 +39,26 @@ class Hamiltonian:
         functionals in Hamiltonian subclasses.
         """
         if not hasattr(self.config, "simulation_volume"):
-            self.config.simulation_volume = np.prod(
-                np.asarray(self.config.box_size)
-            )
-        self.config.rho0 = (
-            self.config.n_particles / self.config.simulation_volume
-        )
-        self.phi = sympy.var("phi:%d" % (len(self.config.unique_names)))
+            self.config.simulation_volume = np.prod(np.asarray(self.config.box_size))
+        if not self.config.barostat:
+            self.config.rho0 = self.config.n_particles / self.config.simulation_volume
+            self.config.a = self.config.rho0
+        if not self.config.rho0:
+            self.config.rho0 = self.config.n_particles / self.config.simulation_volume
+        self.phi = sympy.var("phi:%d" % (self.config.n_types))
         k = sympy.var("k:%d" % (3))
 
+        # electrostatics variables
+        self.psi = sympy.var("psi")
+        self.phi_q = sympy.var("phi_q")
+        if not self.config.self_energy:
+            self.config.self_energy = 0.0
+
         def fourier_space_window_function(k):
             return sympy.functions.elementary.exponential.exp(
                 -0.5
-                * self.config.sigma ** 2
-                * (k0 ** 2 + k1 ** 2 + k2 ** 2)  # noqa: F821, E501
+                * self.config.sigma**2
+                * (k0**2 + k1**2 + k2**2)  # noqa: F821, E501
             )
 
         self.window_function_lambda = sympy.lambdify(
@@ -116,6 +122,7 @@ class SquaredPhi(Hamiltonian):
     --------
     hymd.hamiltonian.DefaultNoChi
     """
+
     def __init__(self, config):
         """Constructor
 
@@ -133,16 +140,61 @@ class SquaredPhi(Hamiltonian):
         self.setup()
 
     def setup(self):
-        """Setup the interaction energy potential and the external potential
-        """
+        """Setup the interaction energy potential and the external potential"""
+
         def w(phi, kappa=self.config.kappa, rho0=self.config.rho0):
             return 0.5 / (kappa * rho0) * (sum(phi)) ** 2
 
+        def w_elec(
+            phi_q,
+            psi,
+            volume=self.config.simulation_volume,
+            self_energy=self.config.self_energy,
+        ):
+            self_energy /= volume
+            return 0.5 * phi_q * psi - self_energy
+
+        def V_bar_0(
+            phi,
+            k,
+            kappa=self.config.kappa,
+            rho0=self.config.rho0,
+        ):
+            V_incompressibility = 1 / (kappa * rho0) * sum(phi)
+            V_interaction = 0
+            return V_interaction + V_incompressibility
+
+        def V_bar_elec(
+            psi,
+            t,
+            type_charges=self.config.type_charges,
+        ):
+            return type_charges[t] * psi
+
+        self.V_bar = [
+            sympy.lambdify(
+                [(self.phi, self.psi)], V_bar_0(self.phi, t) + V_bar_elec(self.psi, t)
+            )
+            for t in range(self.config.n_types)
+        ]
+
         self.v_ext = [
             sympy.lambdify([self.phi], sympy.diff(w(self.phi), self.phi[i]))
-            for i in range(len(self.config.unique_names))
+            for i in range(self.config.n_types)
         ]
-        self.w = sympy.lambdify([self.phi], w(self.phi))
+
+        self.w_0 = sympy.lambdify([self.phi], w(self.phi))
+        self.w_elec = sympy.lambdify(
+            [(self.phi_q, self.psi)], w_elec(self.phi_q, self.psi)
+        )
+
+        if self.config.coulombtype == "PIC_Spectral":
+            self.w = sympy.lambdify(
+                [(self.phi, self.phi_q, self.psi)],
+                w(self.phi) + w_elec(self.phi_q, self.psi),
+            )
+        else:
+            self.w = self.w_0
 
 
 class DefaultNoChi(Hamiltonian):
@@ -153,13 +205,15 @@ class DefaultNoChi(Hamiltonian):
     .. math::
 
         w[\\tilde\\phi] = \\frac{1}{2\\kappa} \\left(
-            \\sum_k \\tilde\\phi_k - \\rho_0
+            \\sum_k \\tilde\\phi_k - a
         \\right)^2,
 
-    where :math:`\\kappa` is the compressibility and :math:`\\rho_0` is the
-    average density of the fully homogenous system. The :code:`SquaredPhi`
-    Hamiltonian implements a similar functional with an additional linear term
-    component depending on
+    where :math:`\\kappa` is the compressibility and :math:`a=\\rho_0` for
+    NVT runs where :math:`\\rho_0` is the average density of the fully
+    homogenous system. In case of NPT runs, :math:`a` is a calibrated
+    parameter to obtain the correct average density at the target temperature
+    and pressure. The :code:`SquaredPhi` Hamiltonian implements a similar
+    functional with an additional linear term component depending on
 
     .. math::
 
@@ -175,6 +229,7 @@ class DefaultNoChi(Hamiltonian):
     hymd.input_parser.Config :
         Configuration dataclass handler.
     """
+
     def __init__(self, config):
         """Constructor
 
@@ -192,16 +247,62 @@ class DefaultNoChi(Hamiltonian):
         self.setup()
 
     def setup(self):
-        """Setup the interaction energy potential and the external potential
-        """
-        def w(phi, kappa=self.config.kappa, rho0=self.config.rho0):
-            return 0.5 / (kappa * rho0) * (sum(phi) - rho0) ** 2
+        """Setup the interaction energy potential and the external potential"""
+
+        def w(phi, kappa=self.config.kappa, rho0=self.config.rho0, a=self.config.a):
+            return 0.5 / (kappa * rho0) * (sum(phi) - a) ** 2
+
+        def w_elec(
+            phi_q,
+            psi,
+            volume=self.config.simulation_volume,
+            self_energy=self.config.self_energy,
+        ):
+            self_energy /= volume
+            return 0.5 * phi_q * psi - self_energy
+
+        def V_bar_0(
+            phi,
+            k,
+            kappa=self.config.kappa,
+            rho0=self.config.rho0,
+            a=self.config.a,
+        ):
+            V_incompressibility = 1 / (kappa * rho0) * (sum(phi) - a)
+            V_interaction = 0
+            return V_interaction + V_incompressibility
+
+        def V_bar_elec(
+            psi,
+            t,
+            type_charges=self.config.type_charges,
+        ):
+            return type_charges[t] * psi
+
+        self.V_bar = [
+            sympy.lambdify(
+                [(self.phi, self.psi)], V_bar_0(self.phi, t) + V_bar_elec(self.psi, t)
+            )
+            for t in range(self.config.n_types)
+        ]
 
         self.v_ext = [
             sympy.lambdify([self.phi], sympy.diff(w(self.phi), self.phi[i]))
-            for i in range(len(self.config.unique_names))
+            for i in range(self.config.n_types)
         ]
-        self.w = sympy.lambdify([self.phi], w(self.phi))
+
+        self.w_0 = sympy.lambdify([self.phi], w(self.phi))
+        self.w_elec = sympy.lambdify(
+            [(self.phi_q, self.psi)], w_elec(self.phi_q, self.psi)
+        )
+
+        if self.config.coulombtype == "PIC_Spectral":
+            self.w = sympy.lambdify(
+                [(self.phi, self.phi_q, self.psi)],
+                w(self.phi) + w_elec(self.phi_q, self.psi),
+            )
+        else:
+            self.w = self.w_0
 
 
 class DefaultWithChi(Hamiltonian):
@@ -216,13 +317,17 @@ class DefaultWithChi(Hamiltonian):
                 \\sum_{k,l}\\chi_{kl} \\tilde\\phi_k \\tilde\\phi_l
             +
             \\frac{1}{2\\kappa} \\left(
-                \\sum_k \\tilde\\phi_k - \\rho_0
+                \\sum_k \\tilde\\phi_k - a
             \\right)^2,
 
-    where :math:`\\kappa` is the incompressibility, :math:`\\rho_0` is the
-    average density of the fully homogenous system and :math:`\\chi_{ij}` is
-    the Flory-Huggins-like inter-species mixing energy.
+    where :math:`\\kappa` is the compressibility and :math:`a=\\rho_0` for
+    NVT runs where :math:`\\rho_0` is the average density of the fully
+    homogenous system. In case of NPT runs, :math:`a` is a calibrated
+    parameter to obtain the correct average density at the target temperature
+    and pressure. :math:`\\chi_{ij}` is the Flory-Huggins-like
+    inter-species mixing energy.
     """
+
     def __init__(self, config, unique_names, type_to_name_map):
         """Constructor
 
@@ -266,17 +371,21 @@ class DefaultWithChi(Hamiltonian):
             tuple(sorted([c.atom_1, c.atom_2])): c.interaction_energy
             for c in self.config.chi
         }
+        self.phi_laplacian = [
+            sympy.var("phi_laplacian%d(0:%d)" % (t, 3))
+            for t in range(self.config.n_types)
+        ]
 
         def w(
             phi,
             kappa=self.config.kappa,
             rho0=self.config.rho0,
+            a=self.config.a,
             chi=self.config.chi,
             type_to_name_map=self.type_to_name_map,
             chi_type_dictionary=self.chi_type_dictionary,
         ):
-
-            interaction = 0
+            interaction = 0.0
             for i in range(self.config.n_types):
                 for j in range(i + 1, self.config.n_types):
                     ni = type_to_name_map[i]
@@ -285,18 +394,79 @@ class DefaultWithChi(Hamiltonian):
                     c = chi_type_dictionary[tuple(names)]
 
                     interaction += c * phi[i] * phi[j] / rho0
-            incompressibility = 0.5 / (kappa * rho0) * (sum(phi) - rho0) ** 2
+            incompressibility = 0.5 / (kappa * rho0) * (sum(phi) - a) ** 2
             return incompressibility + interaction
 
-        self.v_ext = [
-            sympy.lambdify([self.phi], sympy.diff(w(self.phi), self.phi[i]))
-            for i in range(len(self.config.unique_names))
+        def w_elec(
+            phi_q,
+            psi,
+            volume=self.config.simulation_volume,
+            self_energy=self.config.self_energy,
+        ):
+            self_energy /= volume
+            return 0.5 * phi_q * psi - self_energy
+
+        def V_bar_0(
+            phi,
+            t,
+            kappa=self.config.kappa,
+            rho0=self.config.rho0,
+            a=self.config.a,
+            chi=self.config.chi,
+            type_to_name_map=self.type_to_name_map,
+            chi_type_dictionary=self.chi_type_dictionary,
+        ):
+            V_incompressibility = 1 / (kappa * rho0) * (sum(phi) - a)
+
+            V_interaction = 0.0
+            nk = type_to_name_map[t]
+            for i in range(self.config.n_types):
+                ni = type_to_name_map[i]
+                names = sorted([nk, ni])
+                if ni != nk:
+                    c = chi_type_dictionary[tuple(names)]
+                else:
+                    c = 0.0
+                # uncomment to count diagonal chi terms:
+                # c = chi_type_dictionary[tuple(names)]
+                V_interaction += c * phi[i] / rho0
+            return V_interaction + V_incompressibility
+
+        def V_bar_elec(
+            psi,
+            t,
+            type_charges=self.config.type_charges,
+        ):
+            return type_charges[t] * psi
+
+        self.V_bar = [
+            sympy.lambdify(
+                [(self.phi, self.psi)], V_bar_0(self.phi, t) + V_bar_elec(self.psi, t)
+            )
+            for t in range(self.config.n_types)
         ]
-        self.w = sympy.lambdify([self.phi], w(self.phi))
+
+        self.v_ext = [
+            sympy.lambdify([self.phi], sympy.diff(w(self.phi), self.phi[t]))
+            for t in range(self.config.n_types)
+        ]
+
+        self.w_0 = sympy.lambdify([self.phi], w(self.phi))
+        self.w_elec = sympy.lambdify(
+            [(self.phi_q, self.psi)], w_elec(self.phi_q, self.psi)
+        )
+
+        if self.config.coulombtype == "PIC_Spectral":
+            self.w = sympy.lambdify(
+                [(self.phi, self.phi_q, self.psi)],
+                w(self.phi) + w_elec(self.phi_q, self.psi),
+            )
+        else:
+            self.w = self.w_0
 
 
 def get_hamiltonian(config):
-    """Return appropriate Hamiltonian object based on the 
+    """Return appropriate Hamiltonian object based on the
     config.hamiltonian string.
 
     Parameters
@@ -318,4 +488,4 @@ def get_hamiltonian(config):
     elif config.hamiltonian.lower() == "squaredphi":
         hamiltonian = SquaredPhi(config)
 
-    return hamiltonian
\ No newline at end of file
+    return hamiltonian
