@@ -9,7 +9,8 @@ import numpy as np
 from mpi4py import MPI
 from dataclasses import dataclass, field
 from typing import List, Union, ClassVar
-from .force import Bond, Angle, Dihedral, Chi
+from .force import Bond, Angle, Dihedral, Chi, Dielectric_type
+from .barostat import Target_pressure
 from .logger import Logger
 
 
@@ -138,6 +139,12 @@ class Config:
         using helical propensity dihedrals, this keyword must be specified—even
         if electrostatics are not included with the :code:`coulombtype`
         keyword.
+    dielectric_type: list[float], optional
+        Specifies the relative dielectric constant of the simulation medium
+        which regulates the strength of the electrostatic interactions. The list assigns
+        relative dielectric values to each bead type, and an anisotropic
+        dielectric function is obtained from a weighted average.
+
 
     See also
     --------
@@ -148,16 +155,18 @@ class Config:
     hymd.input_parser.Dihedral :
         Four-particle bond type dataclass.
     """
+
     gas_constant: ClassVar[float] = 0.0083144621  # kJ mol-1 K-1
     coulomb_constant: ClassVar[float] = 138.935458  # kJ nm mol-1 e-2
 
     n_steps: int
     time_step: float
-    box_size: Union[List[float], np.ndarray]
     mesh_size: Union[Union[List[int], np.ndarray], int]
     sigma: float
     kappa: float
 
+    dtype: np.dtype = None
+    box_size: Union[List[float], np.ndarray] = None
     n_print: int = None
     tau: float = None
     start_temperature: Union[float, bool] = None
@@ -182,15 +191,35 @@ class Config:
     initial_energy: float = None
     cancel_com_momentum: Union[int, bool] = False
     coulombtype: str = None
+    convergence_type: str = None
+    pol_mixing: float = None
     dielectric_const: float = None
+    conv_crit: float = None
+    dielectric_type: List[Dielectric_type] = field(default_factory=list)
+    self_energy: float = None
+    type_charges: Union[List[float], np.ndarray] = None
+
+    # For NPT runs
+    rho0: float = None
+    a: float = None
+    pressure: bool = False
+    barostat: str = None
+    barostat_type: str = None
+    tau_p: float = None
+    target_pressure: List[Target_pressure] = field(default_factory=list)
+    n_b: int = None
+    m: List[float] = field(default_factory=list)
 
     def __str__(self):
+        target_pressure_str = "\ttarget_pressure:\n" + "".join(
+            "\t\tP_L: "
+            + f"{self.target_pressure.P_L}\n"
+            + "\t\tP_N: "
+            + f"{self.target_pressure.P_N}\n"
+        )
         bonds_str = "\tbonds:\n" + "".join(
             [
-                (
-                    f"\t\t{k.atom_1} {k.atom_2}: "
-                    f"{k.equilibrium}, {k.strength}\n"
-                )
+                (f"\t\t{k.atom_1} {k.atom_2}: " f"{k.equilibrium}, {k.strength}\n")
                 for k in self.bonds
             ]
         )
@@ -211,9 +240,7 @@ class Config:
                     # there's an easier way
                     + (
                         "\n\t\t"
-                        + " " * len(
-                            f"{k.atom_1} {k.atom_2} {k.atom_3} {k.atom_4}: "
-                        )
+                        + " " * len(f"{k.atom_1} {k.atom_2} {k.atom_3} {k.atom_4}: ")
                     ).join(
                         map(
                             str,
@@ -227,9 +254,7 @@ class Config:
                     )
                     + (
                         "\n\t\t"
-                        + " " * len(
-                            f"{k.atom_1} {k.atom_2} {k.atom_3} {k.atom_4}: "
-                        )
+                        + " " * len(f"{k.atom_1} {k.atom_2} {k.atom_3} {k.atom_4}: ")
                     )
                     + f"dih_type = {k.dih_type}\n"
                 )
@@ -242,6 +267,17 @@ class Config:
                 for k in self.chi
             ]
         )
+
+        """ If dielectric wanted as dictionary
+            dielectric_type_str = "\tdielectric_type:\n" + "".join(
+                    [
+                        (f"\t\t{k.atom_1}: " + f"{k.dielectric_value}\n")
+                        for k in self.dielectric_type
+                    ]
+                )
+
+        """
+
         thermostat_coupling_groups_str = ""
         if any(self.thermostat_coupling_groups):
             thermostat_coupling_groups_str = (
@@ -257,6 +293,7 @@ class Config:
         ret_str = f'\n\n\tConfig: {self.file_name}\n\t{50 * "-"}\n'
         for k, v in self.__dict__.items():
             if k not in (
+                "target_pressure",
                 "bonds",
                 "angle_bonds",
                 "dihedrals",
@@ -265,7 +302,8 @@ class Config:
             ):
                 ret_str += f"\t{k}: {v}\n"
         ret_str += (
-            bonds_str
+            target_pressure_str
+            + bonds_str
             + angle_str
             + dihedrals_str
             + chi_str
@@ -276,9 +314,8 @@ class Config:
 
 def read_config_toml(file_path):
     with open(file_path, "r") as in_file:
-        toml_content = in_file.read()
-    return toml_content
-
+        file_content = in_file.read()
+    return file_content
 
 def propensity_potential_coeffs(x: float, comm):
     alpha_coeffs = np.array(
@@ -329,11 +366,14 @@ def propensity_potential_coeffs(x: float, comm):
 
 
 def parse_config_toml(toml_content, file_path=None, comm=MPI.COMM_WORLD):
-    parsed_toml = tomli.loads(toml_content)
     config_dict = {}
 
+    # read toml to a dictionary
+    toml_content = tomli.loads(toml_content)
+
     # Defaults = None
     for n in (
+        "box_size",
         "n_print",
         "tau",
         "start_temperature",
@@ -346,19 +386,43 @@ def parse_config_toml(toml_content, file_path=None, comm=MPI.COMM_WORLD):
         "n_particles",
         "max_molecule_size",
         "coulombtype",
+        "convergence_type",
         "dielectric_const",
+        "pol_mixing",
+        "conv_crit",
+        "dielectric_type",
+        "n_b",
+        "box_size",
         "n_flush",
+        "self_energy",
+        "dtype",
     ):
         config_dict[n] = None
 
     # Defaults = []
-    for n in ("bonds", "angle_bonds", "dihedrals", "chi", "tags"):
+
+    for n in (
+        "bonds",
+        "angle_bonds",
+        "dihedrals",
+        "chi",
+        "tags",
+        "m",
+        "dielectric_type",
+        "target_pressure",
+        "type_charges",
+    ):
         config_dict[n] = []
 
+    # Defaults: bool
+    config_dict["pressure"] = False
+
     # Flatten the .toml dictionary, ignoring the top level [tag] directives (if
     # any).
-    for k, v in parsed_toml.items():
+    for k, v in toml_content.items():
         if isinstance(v, dict):
+            if k == "nn": # Don't parse diff-hymd optimization options
+                continue
             for nested_k, nested_v in v.items():
                 config_dict[nested_k] = nested_v
         else:
@@ -390,19 +454,14 @@ def parse_config_toml(toml_content, file_path=None, comm=MPI.COMM_WORLD):
                     dih_type = int(b[2][0])
                 except IndexError:
                     Logger.rank0.log(
-                        logging.WARNING,
-                        "Dihedral type not provided, defaulting to 0."
+                        logging.WARNING, "Dihedral type not provided, defaulting to 0."
                     )
                     dih_type = 0
 
                     # Probably it's better to move this in check_dihedrals?
                     wrong_len = len(b[1]) not in (1, 2)
-                    wrong_type_1 = len(b[1]) == 1 and not isinstance(
-                        b[1][0], float
-                    )
-                    wrong_type_2 = len(b[1]) == 2 and not isinstance(
-                        b[1][0], list
-                    )
+                    wrong_type_1 = len(b[1]) == 1 and not isinstance(b[1][0], float)
+                    wrong_type_2 = len(b[1]) == 2 and not isinstance(b[1][0], list)
                     if wrong_len or wrong_type_1 or wrong_type_2:
                         err_str = (
                             "The coefficients specified for the dihedral type "
@@ -425,9 +484,7 @@ def parse_config_toml(toml_content, file_path=None, comm=MPI.COMM_WORLD):
                 elif dih_type == 2:
                     coeff = np.array(b[1])
                 else:
-                    coeff = np.insert(
-                        np.array(b[1]), 2, np.zeros((2, 5)), axis=0
-                    )
+                    coeff = np.insert(np.array(b[1]), 2, np.zeros((2, 5)), axis=0)
 
                 config_dict["dihedrals"][i] = Dihedral(
                     atom_1=b[0][0],
@@ -437,9 +494,6 @@ def parse_config_toml(toml_content, file_path=None, comm=MPI.COMM_WORLD):
                     coeffs=coeff,
                     dih_type=dih_type,
                 )
-        # if k == "improper dihedrals":
-        #     config_dict["improper dihedrals"] = [None] * len(v)
-        # ...
         if k == "chi":
             config_dict["chi"] = [None] * len(v)
             for i, c in enumerate(v):
@@ -448,11 +502,31 @@ def parse_config_toml(toml_content, file_path=None, comm=MPI.COMM_WORLD):
                     atom_1=c_[0], atom_2=c_[1], interaction_energy=c[2]
                 )
 
+        """
+        if k == "dielectric_type":
+            config_dict["dielectric_type"] = [None] * len(v)
+            for i, c in enumerate(v):
+                c_ = sorted([c[0][0]])
+                config_dict["dielectric_type"][i] = Dielectric_type(
+                    atom_1=c_[0], dielectric_value=c[1][0]
+                )
+        """
+        if k == "target_pressure":
+            if len(v) == 2:
+                config_dict["target_pressure"] = Target_pressure(
+                    P_L=v[0],
+                    P_N=v[1],  # check condition # V array (still read as array?)
+                )
+            elif len(v) == 1:
+                config_dict["target_pressure"] = Target_pressure(P_L=v[0], P_N=None)
+            else:
+                config_dict["target_pressure"] = Target_pressure(P_L=None, P_N=None)
+
     if file_path is not None:
         config_dict["file_name"] = file_path
 
     for n in (
-        "n_steps", "time_step", "box_size", "mesh_size", "sigma", "kappa"
+        "n_steps", "time_step", "mesh_size", "sigma", "kappa"
     ):
         if n not in config_dict:
             err_str = (
@@ -639,9 +713,7 @@ def check_dihedrals(config, names, comm=MPI.COMM_WORLD):
             ]
             missing_names = [
                 atom
-                for i, atom in enumerate(
-                    [d.atom_1, d.atom_2, d.atom_3, d.atom_4]
-                )
+                for i, atom in enumerate([d.atom_1, d.atom_2, d.atom_3, d.atom_4])
                 if missing[i]
             ]
             missing_str = ", ".join(np.unique(missing_names))
@@ -686,7 +758,7 @@ def check_chi(config, names, comm=MPI.COMM_WORLD):
                 warnings.warn(warn_str)
 
     for i, n in enumerate(unique_names):
-        for m in unique_names[i+1:]:
+        for m in unique_names[i + 1 :]:
             found = False
             for c in config.chi:
                 if (c.atom_1 == n and c.atom_2 == m) or (
@@ -694,9 +766,7 @@ def check_chi(config, names, comm=MPI.COMM_WORLD):
                 ):
                     found = True
             if not found:
-                config.chi.append(
-                    Chi(atom_1=n, atom_2=m, interaction_energy=0.0)
-                )
+                config.chi.append(Chi(atom_1=n, atom_2=m, interaction_energy=0.0))
                 warn_str = (
                     f"Atom types {n} and {m} found in the "
                     f"system, but no chi interaction {n}--{m} "
@@ -709,7 +779,27 @@ def check_chi(config, names, comm=MPI.COMM_WORLD):
     return config
 
 
-def check_box_size(config, comm=MPI.COMM_WORLD):
+def check_box_size(config, input_box, comm=MPI.COMM_WORLD):
+    if config.box_size is not None:
+        config.box_size = np.array(config.box_size, dtype=np.float32)
+        if input_box.all() and not np.allclose(config.box_size, input_box, atol=0.009):
+            err_str = (
+                f"Box size specified in {config.file_name}: "
+                f"{config.box_size} does not match input box:"
+                f"{input_box}"
+            )
+            Logger.rank0.log(logging.ERROR, err_str)
+            if comm.Get_rank() == 0:
+                raise ValueError(err_str)
+    else:
+        if input_box.all():
+            config.box_size = input_box
+        else:
+            err_str = f"No box information found"
+            Logger.rank0.log(logging.ERROR, err_str)
+            if comm.Get_rank() == 0:
+                raise ValueError(err_str)
+
     for b in config.box_size:
         if b <= 0.0:
             err_str = (
@@ -719,7 +809,6 @@ def check_box_size(config, comm=MPI.COMM_WORLD):
             Logger.rank0.log(logging.ERROR, err_str)
             if comm.Get_rank() == 0:
                 raise ValueError(err_str)
-    config.box_size = np.array(config.box_size, dtype=np.float64)
     return config
 
 
@@ -763,10 +852,7 @@ def check_integrator(config, comm=MPI.COMM_WORLD):
             Logger.rank0.log(logging.ERROR, err_str)
             raise ValueError(err_str)
 
-    if (
-        config.integrator.lower() == "velocity-verlet"
-        and config.respa_inner != 1
-    ):
+    if config.integrator.lower() == "velocity-verlet" and config.respa_inner != 1:
         warn_str = (
             f"Integrator type Velocity-Verlet specified in {config.file_name} "
             f"and inner rRESPA time steps set to {config.respa_inner}. "
@@ -822,15 +908,15 @@ def check_n_flush(config, comm=MPI.COMM_WORLD):
 
 
 def check_n_print(config, comm=MPI.COMM_WORLD):
-    if (not isinstance(config.n_print, int) and
-        not isinstance(config.n_print, float) and
-        config.n_print is not None):
-        err_str = (
-            f"invalid value for n_print ({config.n_print})"
-        )
+    if (
+        not isinstance(config.n_print, int)
+        and not isinstance(config.n_print, float)
+        and config.n_print is not None
+    ):
+        err_str = f"invalid value for n_print ({config.n_print})"
         Logger.rank0.log(logging.ERROR, err_str)
-        raise RuntimeError(err_str)        
-    
+        raise RuntimeError(err_str)
+
     if config.n_print is None or config.n_print <= 0:
         config.n_print = False
     elif not isinstance(config.n_print, int):
@@ -845,9 +931,7 @@ def check_n_print(config, comm=MPI.COMM_WORLD):
 
             config.n_print = int(round(config.n_print))
         else:
-            err_str = (
-                f"invalid value for n_print ({config.n_print})"
-            )
+            err_str = f"invalid value for n_print ({config.n_print})"
             Logger.rank0.log(logging.ERROR, err_str)
             raise RuntimeError(err_str)
     return config
@@ -917,8 +1001,7 @@ def check_mass(config, comm=MPI.COMM_WORLD):
         Logger.rank0.log(logging.INFO, info_str)
     elif not isinstance(config.mass, int) and not isinstance(config.mass, float):
         err_str = (
-            f"specified mass is invalid type {config.mass}, "
-            f"({type(config.mass)})"
+            f"specified mass is invalid type {config.mass}, " f"({type(config.mass)})"
         )
         Logger.rank0.log(logging.ERROR, err_str)
         raise TypeError(err_str)
@@ -959,8 +1042,10 @@ def check_domain_decomposition(config, comm=MPI.COMM_WORLD):
         if comm.Get_rank() == 0:
             warnings.warn(warn_str)
     else:
-        err_str = (f"invalid value for domain_decomposition "
-                   f"({config.domain_decomposition}) use an integer")
+        err_str = (
+            f"invalid value for domain_decomposition "
+            f"({config.domain_decomposition}) use an integer"
+        )
         Logger.rank0.log(logging.ERROR, err_str)
         raise ValueError(err_str)
 
@@ -971,9 +1056,7 @@ def check_name(config, comm=MPI.COMM_WORLD):
     if config.name is None:
         root_current_time = ""
         if comm.Get_rank() == 0:
-            root_current_time = datetime.datetime.now().strftime(
-                "%m/%d/%Y, %H:%M:%S"
-            )
+            root_current_time = datetime.datetime.now().strftime("%m/%d/%Y, %H:%M:%S")
         current_time = comm.bcast(root_current_time, root=0)
 
         if config.name is None:
@@ -983,6 +1066,84 @@ def check_name(config, comm=MPI.COMM_WORLD):
     return config
 
 
+def check_NPT_conditions(config, comm=MPI.COMM_WORLD):
+    """
+    Check validity of barostat_type, barostat,
+    a, rho0, target_pressure, tau_p
+    """
+    if config.barostat is None:
+        if (
+            config.tau_p is not None
+            or (config.target_pressure.P_L and config.target_pressure.P_N) is not None
+        ):
+            err_str = (
+                "barostat not specified but config.tau_p "
+                "or config.target_pressure specified, cannot start simulation {config.barostat}"
+            )
+            Logger.rank0.log(logging.ERROR, err_str)
+            if comm.Get_rank() == 0:
+                raise TypeError(err_str)
+        if config.a:
+            warn_str = "a specified but no barostat," "setting a to average density"
+            Logger.rank0.log(logging.WARNING, warn_str)
+            if comm.Get_rank() == 0:
+                warnings.warn(warn_str)
+        if config.rho0:
+            warn_str = (
+                "rho0 specified but no barostat," "setting rho0 to average density"
+            )
+            Logger.rank0.log(logging.WARNING, warn_str)
+            if comm.Get_rank() == 0:
+                warnings.warn(warn_str)
+
+    if config.barostat is not None:
+        if not config.barostat_type:
+            config.barostat_type = "berendsen"
+            warn_str = "barostat_type not specified," "setting to berendsen"
+            Logger.rank0.log(logging.WARNING, warn_str)
+            if comm.Get_rank() == 0:
+                warnings.warn(warn_str)
+        if config.barostat != "isotropic" and config.barostat != "semiisotropic":
+            err_str = "barostat option not recognised. Valid options: isotropic, semiisotropic"
+            Logger.rank0.log(logging.ERROR, err_str)
+            if comm.Get_rank() == 0:
+                raise TypeError(err_str)
+        if config.target_pressure.P_L is None:
+            config.target_pressure.P_L = 1.0  # bar
+            config.target_pressure.P_N = None
+            if config.barostat == "semiisotropic":
+                config.target_pressure.P_N = 1.0  # bar
+            warn_str = (
+                "barostat specified but no target_pressure, defaulting to 1.0 bar"
+            )
+            Logger.rank0.log(logging.WARNING, warn_str)
+            if comm.Get_rank() == 0:
+                warnings.warn(warn_str)
+        if config.tau_p is None:
+            if config.tau <= 0.1:
+                config.tau_p = config.tau * 10.0
+            else:
+                config.tau_p = 1.0
+            warn_str = "barostat specified but no tau_p, defaulting to " + str(
+                config.tau_p
+            )
+            Logger.rank0.log(logging.WARNING, warn_str)
+            if comm.Get_rank() == 0:
+                warnings.warn(warn_str)
+        if not config.a:
+            err_str = "a not specified; cannot start simulation {config.a}"
+            Logger.rank0.log(logging.ERROR, err_str)
+            if comm.Get_rank() == 0:
+                raise TypeError(err_str)
+        if not config.rho0:
+            warn_str = "rho0 not specified;" "setting rho0 to average density"
+            Logger.rank0.log(logging.WARNING, warn_str)
+            if comm.Get_rank() == 0:
+                warnings.warn(warn_str)
+
+    return config
+
+
 def check_thermostat_coupling_groups(config, comm=MPI.COMM_WORLD):
     if any(config.thermostat_coupling_groups):
         found = [0 for _ in config.unique_names]
@@ -1022,6 +1183,22 @@ def check_thermostat_coupling_groups(config, comm=MPI.COMM_WORLD):
     return config
 
 
+def check_m(config, comm=MPI.COMM_WORLD):
+    if config.m == []:
+        config.m = [1.0 for t in range(config.n_types)]
+    return config
+
+
+def check_n_b(config, comm=MPI.COMM_WORLD):
+    if config.n_b is None:
+        warn_str = f"config.n_b not specified." "Defaulting to 1"
+        config.n_b = 1
+        Logger.rank0.log(logging.WARNING, warn_str)
+        if comm.Get_rank() == 0:
+            warnings.warn(warn_str)
+    return config
+
+
 def check_cancel_com_momentum(config, comm=MPI.COMM_WORLD):
     if isinstance(config.cancel_com_momentum, int):
         if config.cancel_com_momentum == 0 or config.cancel_com_momentum < 0:
@@ -1035,7 +1212,138 @@ def check_cancel_com_momentum(config, comm=MPI.COMM_WORLD):
     return config
 
 
-def check_config(config, indices, names, types, comm=MPI.COMM_WORLD):
+def sort_dielectric_by_type_id(config, charges, types):
+    """
+    Creates a list of length N CG-particles, sorted after the charges from
+    the input HDF5 file. Used in file_io.py
+    """
+    dielectric_val = np.zeros(config.n_types)
+    for j in range(config.n_types):
+        name = config.dielectric_type[j][0][0]
+        type_id = config.name_to_type_map[name]
+        dielectric_val[type_id] = config.dielectric_type[j][1][0]
+    config.dielectric_type = dielectric_val.copy()
+
+    N = len(charges)
+    len_list = np.zeros(config.n_types)
+    dielectric_by_types = np.zeros(N)
+    for i in range(N):
+        dielectric_by_types[i] = dielectric_val[types[i]]
+    # print("types ", types)
+    # print("diel  val", dielectric_by_types)
+    # print("charges", charges)
+    # print(config.name_to_type_map)
+    return dielectric_by_types  # by types with each particle id
+
+
+def check_charges_types_list(config, types, charges, comm=MPI.COMM_WORLD):
+    """
+    Creates a list of charge values of length types.
+    Charges are sorted according to type ID. Used in field.py.
+    # TODO: this is messy, we should fix it
+    """
+    if charges is None:
+        config.type_charges = [0.0] * config.n_types
+        return config
+
+    check_val = -100.0  # some random value that will never be encountered
+    charges_list = np.full(config.n_types, check_val)  # gatherv cant handle None
+    rank = comm.Get_rank()
+    # print(charges_list)
+    for t_ in range(config.n_types):
+        if t_ in types:
+            charges_list[t_] = charges[types == t_][0]
+
+    nprocs = int(comm.Get_size())
+    recv_charges = None
+    if rank == 0:
+        recv_charges = np.full(
+            config.n_types * nprocs, check_val
+        )  # gatherv cant handle None
+    comm.Gather(charges_list, recv_charges, root=0)
+
+    ## make a charges list
+    if rank == 0:
+        config_charges = np.zeros(config.n_types)
+        test_config = np.full(config.n_types, check_val)
+        for j in range(nprocs):
+            for i in range(config.n_types):
+                if recv_charges[i + j * config.n_types] != check_val:
+                    config_charges[i] = recv_charges[i + j * config.n_types]
+            if np.any([test_config, config_charges]):
+                continue
+            else:
+                break
+
+        # print(config_charges)
+        # print(config.name_to_type_map)
+        # print(charges[types == 0][0],charges[types == 1][0],charges[types == 2][0],charges[types == 3][0],
+        #        charges[types == 4][0])
+    else:
+        config_charges = np.zeros(config.n_types)
+
+    comm.Bcast(config_charges, root=0)
+    # print("config_charges", config_charges , "rank", rank)
+    # print(recv_charges)
+    # rank = comm.Get_rank()
+    # print(all_charges)
+    config.type_charges = config_charges
+    return config
+
+
+def check_dielectric(config, comm=MPI.COMM_WORLD):
+    """
+    Error handling for electrostatics.
+    Unit testing of toml/tomli input.
+    """
+    err_str_const = "Dielectric constant not given."
+    if config.coulombtype == "PIC_Spectral":
+        assert config.dielectric_const != None, err_str_const
+
+    err_str = "Dielectric list is empty."
+    if config.coulombtype == "PIC_Spectral_GPE":
+        assert len(config.dielectric_type) != 0, err_str
+        # default values
+        if config.pol_mixing is None:
+            config.pol_mixing = 0.6
+        if config.conv_crit is None:
+            config.conv_crit = 1e-6
+    return config
+
+
+def check_charges(config, charges, comm=MPI.COMM_WORLD):
+    """Check if charges across ranks sum to zero.
+
+    Parameters
+    ----------
+    charges : (N,) numpy.ndarray
+        Array of floats with charges for :code:`N` particles.
+    comm : mpi4py.Comm, optional
+        MPI communicator, defaults to :code:`mpi4py.COMM_WORLD`.
+    """
+    total_charge = comm.allreduce(np.sum(charges), MPI.SUM)
+
+    if not np.isclose(total_charge, 0.0):
+        warn_str = (
+            f"Charges in the input file do not sum to zero. "
+            f"Total charge is {total_charge}."
+        )
+        Logger.rank0.log(logging.WARNING, warn_str)
+        if comm.Get_rank() == 0:
+            warnings.warn(warn_str)
+
+    # if charges are ok, compute self energy
+    if config.coulombtype == "PIC_Spectral":
+        from .field import compute_self_energy_q
+
+        config.self_energy = compute_self_energy_q(config, charges, comm=comm)
+
+    return config
+
+
+def check_config(
+    config, indices, names, types, charges, input_box, comm=MPI.COMM_WORLD
+):
     """Performs various checks on the specfied config to ensure consistency
 
     Parameters
@@ -1048,6 +1356,8 @@ def check_config(config, indices, names, types, comm=MPI.COMM_WORLD):
         Array of string names for :code:`N` particles.
     types : (N,) numpy.ndarray
         Array of integer type indices for :code:`N` particles.
+    charges : (N,) numpy.ndarray
+        Array of floats charges for :code:`N` particles.
     comm : mpi4py.Comm, optional
         MPI communicator, defaults to :code:`mpi4py.COMM_WORLD`.
 
@@ -1056,11 +1366,10 @@ def check_config(config, indices, names, types, comm=MPI.COMM_WORLD):
     config : Config
         Validated configuration object.
     """
-    config.box_size = np.array(config.box_size)
     config = _find_unique_names(config, names, comm=comm)
     if types is not None:
         config = _setup_type_to_name_map(config, names, types, comm=comm)
-    config = check_box_size(config, comm=comm)
+    config = check_box_size(config, input_box, comm=comm)
     config = check_integrator(config, comm=comm)
     config = check_max_molecule_size(config, comm=comm)
     config = check_tau(config, comm=comm)
@@ -1074,31 +1383,16 @@ def check_config(config, indices, names, types, comm=MPI.COMM_WORLD):
     config = check_angles(config, names, comm=comm)
     config = check_dihedrals(config, names, comm=comm)
     config = check_hamiltonian(config, comm=comm)
+    config = check_NPT_conditions(config, comm=comm)
+    config = check_n_b(config, comm=comm)
+    config = check_m(config, comm=comm)
     config = check_thermostat_coupling_groups(config, comm=comm)
     config = check_cancel_com_momentum(config, comm=comm)
+    config = check_dielectric(config, comm=comm)
     config = check_n_print(config, comm=comm)
     config = check_n_flush(config, comm=comm)
+    config = check_charges_types_list(config, types, charges, comm=comm)
+    if charges is not None:
+        config = check_charges(config, charges, comm=comm)
+
     return config
-
-
-def check_charges(charges, comm=MPI.COMM_WORLD):
-    """Check if charges across ranks sum to zero.
-
-    Parameters
-    ----------
-    charges : (N,) numpy.ndarray
-        Array of floats with charges for :code:`N` particles.
-    comm : mpi4py.Comm, optional
-        MPI communicator, defaults to :code:`mpi4py.COMM_WORLD`.
-    """  
-    total_charge = comm.allreduce(np.sum(charges), MPI.SUM)
-
-    if not np.isclose(total_charge, 0.):
-        warn_str = (
-            f"Charges in the input file do not sum to zero. "
-            f"Total charge is {total_charge}."
-        )
-        Logger.rank0.log(logging.WARNING, warn_str)
-        if comm.Get_rank() == 0:
-            warnings.warn(warn_str)
-
